#include <Eigen/Core>
#include <iostream>
#include "writer.hpp"
#include <cmath>
#include <stdexcept>

//! Vector type
typedef Eigen::VectorXd Vector;

/// Uses forward Euler and upwind finite differences to compute u from time 0 to time T 
///
/// @param[out] u solution at all time steps up to time T, each column corresponding to a time step (including the initial condition as first column)
/// @param[out] time the time levels
/// @param[in] u0 the initial conditions, as column vector
/// @param[in] dt the time step size
/// @param[in] T the final time at which to compute the solution (which we assume to be a multiple of dt)
/// @param[in] N the number of grid points, INCLUDING BOUNDARY POINTS
/// @param[in] a the advection velocity
///

void UpwindFD(Eigen::MatrixXd & u,  Vector & time, const Vector u0, double dt, double T, int N, double a)
{
  const unsigned int nsteps = round(T/dt);
  const double dx = 1./(N-1);
  u.resize(N,nsteps+1);
  time.resize(nsteps+1);

  /* Initialize u */
 

  time[0]=0;

  /* Implement the timestepping scheme with upwind finite differences */


}

/// Uses forward Euler and downwind finite differences to compute u from time 0 to time T 
///
/// @param[out] u solution at all time steps up to time T, each column corresponding to a time step (including the initial condition as first column)
/// @param[out] time the time levels
/// @param[in] u0 the initial conditions, as column vector
/// @param[in] dt the time step size
/// @param[in] T the final time at which to compute the solution (which we assume to be a multiple of dt)
/// @param[in] N the number of grid points, INCLUDING BOUNDARY POINTS
/// @param[in] a the advection velocity
///

void DownwindFD(Eigen::MatrixXd & u,  Vector & time, const Vector u0, double dt, double T, int N, double a)
{
  const unsigned int nsteps = round(T/dt);
  const double dx = 1./(N-1);
  u.resize(N,nsteps+1);
  time.resize(nsteps+1);

  /* Initialize u */


  time[0]=0;

  /* Implement the timestepping scheme with upwind finite differences */


}

/// Uses forward Euler and centered finite differences to compute u from time 0 to time T 
///
/// @param[out] u solution at all time steps up to time T, each column corresponding to a time step (including the initial condition as first column)
/// @param[out] time the time levels
/// @param[in] u0 the initial conditions, as column vector
/// @param[in] dt the time step size
/// @param[in] T the final time at which to compute the solution (which we assume to be a multiple of dt)
/// @param[in] N the number of grid points, INCLUDING BOUNDARY POINTS
/// @param[in] a the advection velocity
///

void CenteredFD(Eigen::MatrixXd & u,  Vector & time, const Vector u0, double dt, double T, int N, double a)
{
  const unsigned int nsteps = round(T/dt);
  const double dx = 1./(N-1);
  u.resize(N,nsteps+1);
  time.resize(nsteps+1);

  /* Initialize u */


  time[0]=0;

  /* Implement the timestepping scheme with upwind finite differences */


}

/* Initial condition: rectangle */
double U0(double x) { 
  if(x<0.25 || x> 0.75)
    return 0;
  else
    return 2.;
}

int main(int, char**) {
    double T = 2.;
    double dt = 0.01; // Change this for timestep comparison
    int N=101;
    double a = 1.; // Change this for changing the velocity
    Vector u0(N);
    double h=1./(N-1);
    /* Initialize u0 */
    for(int i=0;i<u0.size();i++)
      u0[i] = U0(h*(double)i);

    Eigen::MatrixXd u_upwind;
    Vector time_upwind;
    UpwindFD(u_upwind,time_upwind,u0,dt,T,N,a);
    writeToFile("time_upwind.txt",time_upwind);
    writeMatrixToFile("u_upwind.txt", u_upwind);

    Eigen::MatrixXd u_downwind;
    Vector time_downwind;
    DownwindFD(u_downwind,time_downwind,u0,dt,T,N,a);
    writeToFile("time_downwind.txt",time_downwind);
    writeMatrixToFile("u_downwind.txt", u_downwind);

    Eigen::MatrixXd u_centered;
    Vector time_centered;
    DownwindFD(u_centered,time_centered,u0,dt,T,N,a);
    writeToFile("time_centered.txt",time_centered);
    writeMatrixToFile("u_centered.txt", u_centered);
}
